#!/usr/bin/python
# -*- coding: latin-1 -*-

##
#   Project: pybrowser - a faster and lightweight web browser written in Python with a GTK+ gui
#   Author: Mario Guerriero <mefrio.g@gmail.com>
#   Copyright: 2011 Mario Guerriero
#   License: GPL-3+
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation; either version 3 of the License, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful, but WITHOUT
#  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
# On Debian GNU/Linux systems, the full text of the GNU General Public License
# can be found in the file /usr/share/common-licenses/GPL-3.

try:
	import pygtk
	import gtk
	import gtk.glade
except:
	print "Install pygtk, gtk and glade modules"
	exit()

try:
	import urllib
	import os
	from sys import argv
	from ConfigParser import RawConfigParser
	from time import sleep
	from shutil import copyfile
except:
	print "Install os, sys, urllib, sleep, shutil and ConfigParser modules but...do you have Python? :-)"  
	exit()

try:
	import webkit
except:
	print "Install webkit module for python"
	exit()
	
import settings	
import bookmarks

class pyBrowser:
	def __init__(self, url=""):
		self.key_sum = ""
		
		def on_wndMain_key_press_event(widget, event, data=None):			
			keyname = gtk.gdk.keyval_name(event.keyval)
			
			self.key_sum += (keyname + "|")

			key_split_sum = self.key_sum.split("|")
			
			'''
			if len(key_split_sum) == 3:
				self.key_sum = []
			'''
			if keyname == "F5":				#Refresh 
				self.Browser.reload()
				
			if keyname == "F11":			#Full screen
				if self.fullscreen == 0:
					self.wndMain.fullscreen()
					self.fullscreen = 1
				elif self.fullscreen == 1:
					self.wndMain.unfullscreen()
					self.fullscreen = 0
			
			if key_split_sum[-2] == "plus" and key_split_sum[-3] == "Control_L":		#set zoom
				self.Browser.zoom_in()
			elif key_split_sum[-2] == "minus" and key_split_sum[-3] == "Control_L":
				self.Browser.zoom_out()
			elif key_split_sum[-2] == "0" and key_split_sum[-3] == "Control_L":
				self.Browser.set_zoom_level(1)

		def on_wndMain_button_press_event(widget, event, data=None):
			#print event
			#button_name = gtk.gdk.button_name(event.button)
			pass
					
		self.MainGlade = gtk.glade.XML("../ui/pybrowser.glade")
		
		gw = self.MainGlade.get_widget
		
		self.wndMain = gw("wndMain")
		self.browView = gw("browView")
		self.entUrl = gw("entUrl")
		self.btnBack = gw("btnBack")
		self.btnForward = gw("btnForward")
		self.imgStopRefresh = gw("imgStopRefresh") 

		self.btnSettings = gw("btnSettings")
		self.menu = gw("menu")
				
		self.fullscreen = 0
		
		self.controll_folder()
		
		config = RawConfigParser()
		config.read("/home/"+os.getlogin()+"/.pybrowser/settings.ini")
		self.url = config.get("General", "Homepage")
		
		self.entUrl.set_text(self.url)
		
		self.start_web(self.url)
	
		self.controll_web_status()
		
		signals = {
			"on_wndMain_destroy":self.on_wndMain_destroy,
			"on_btnHome_clicked":self.on_btnHome_clicked,
			"on_btnGoUrl_clicked":self.on_btnGoUrl_clicked,
			"on_btnBack_clicked":self.on_btnBack_clicked,
			"on_btnForward_clicked":self.on_btnForward_clicked,
			"on_btnStopRefresh_clicked":self.on_btnStopRefresh_clicked,
			"on_btnSettings_clicked":self.on_btnSettings_clicked,
			"on_wndMain_key_press_event":on_wndMain_key_press_event,
			"on_wndMain_button_press_event":on_wndMain_button_press_event,
			"on_entUrl_activate":self.on_btnGoUrl_clicked,
			"on_menuSettings_activate":self.on_menuSettings_activate,
			"on_menuAddBookmark_activate":self.on_menuAddBookmark_activate,
			"on_menuBookmark_activate":self.on_menuBookmark_activate,
			"on_menu_deactivate":self.on_menu_deactivate
		}
		
		self.MainGlade.signal_autoconnect(signals)
		
		self.Browser.connect("download-requested", self.download_requested)
		self.Browser.connect("load-finished", self.load_finished)
		self.Browser.connect("load-started", self.load_started)
		self.Browser.connect("load-progress-changed", self.load_progress_changed)
		self.Browser.connect("load-error", self.load_error)
	
	#events functions
	def load_started(self, view, frame):
		self.StopRefresh = 0
		self.wndMain.set_title("Loading...")
		self.imgStopRefresh.set_from_stock("gtk-stop", "large-toolbar")

	def load_progress_changed(self, view, frame):
		self.entUrl.set_progress_fraction(self.Browser.get_progress())

	def load_error(self, view, frame, web, data=None):
		self.refresh_window()
		self.set_entry_url()
		self.entUrl.set_progress_fraction(self.Browser.get_progress())
					
	def load_finished(self, view, frame):
		self.StopRefresh = 1
		self.refresh_window()
		self.set_entry_url()
		self.imgStopRefresh.set_from_stock("gtk-refresh", "large-toolbar")
		self.entUrl.set_progress_fraction(1)
		sleep(1)
		self.entUrl.set_progress_fraction(0)

		
	def download_requested(self, view, frame):
		print frame, view
		try:
			download_url = self.Browser.get_focused_frame().get_uri()
			file_name = download_url[0:(len(download_url)-1)]
			file_name = file_name.replace("http://", "")
			file_download = file_name.split("/")
			print file_download[-1]
			urllib.urlretrieve(download_url, "/home/" + os.getlogin() + "/" + file_name)
		except:
			print "Cannot download the file"
	
	#window/others settings
	def controll_web_status(self):
		if not self.Browser.can_go_forward():
			self.btnForward.set_sensitive(False)
		else:
			self.btnForward.set_sensitive(True)
	
	def refresh_window(self):
		self.controll_web_status()
		self.set_window_title()
		self.set_entry_url()
		
	def set_window_title(self):
		self.wndMain.set_title( self.Browser.get_main_frame().get_title() + " - pyBrowser" )
	
	def set_entry_url(self):
		self.entUrl.set_text( self.Browser.get_main_frame().get_uri() )
							
	def start_web(self, url):
		self.Browser = webkit.WebView()
		if url[0:7] != "http://":
			url_mod = url
			url = ""
			url = "http://" + url_mod
		elif url[0:7] == "http://":
			pass
		self.Browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		self.Browser.open(url)
		self.Browser.execute_script('document.title=oldtitle;')
		self.controll_web_status()
		self.browView.add(self.Browser)
	
	def controll_folder(self):
		if os.path.exists("/home/"+os.getlogin()+"/.pybrowser"):
			pass
		else:
			os.mkdir("/home/"+os.getlogin()+"/.pybrowser")
			copyfile("../data/settings.ini", "/home/"+os.getlogin()+"/.pybrowser/settings.ini")
			copyfile("../data/bookmark.ini", "/home/"+os.getlogin()+"/.pybrowser/bookmark.ini")
	
	#buttons and window
	def on_wndMain_destroy(self, data=None):
		gtk.main_quit()
			
	def on_btnHome_clicked(self, data=None):
		self.Browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		self.Browser.open("http://www.google.com")
		self.Browser.execute_script('document.title=oldtitle;')
		
	def on_btnGoUrl_clicked(self, data=None):
		url = self.entUrl.get_text()
		if url[0:7] != "http://":
			url_mod = url
			url = ""
			url = "http://" + url_mod
		elif url[0:7] == "http://":
			pass
		self.Browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		self.Browser.open(url)
		self.Browser.execute_script('document.title=oldtitle;')

	
	def on_btnBack_clicked(self, data=None):
		self.Browser.go_back()	
				
	def on_btnForward_clicked(self, data=None):
		self.Browser.go_forward()
		
	def on_btnStopRefresh_clicked(self, data=None):
		if self.StopRefresh == 0:
			self.Browser.stop_loading()
			self.controll_web_status()
		elif self.StopRefresh == 1:	
			self.Browser.reload()
			self.controll_web_status()
			
	def on_btnSettings_clicked(self, widget):
		if not self.menu.get_attach_widget():
			self.menu.attach_to_widget(widget, None)
		self.menu.popup(None, None, self.menu_position, 1, gtk.get_current_event_time())
		self.menu.select_first(True)
		self.btnSettings.set_sensitive(False)
		
	def on_entUrl_activate(self, data=None):
		self.on_btnGoUrl_clicked()
	
	
	#functions for menu
	def menu_position(self, menu, data=None):
		widget = menu.get_attach_widget()
		allocation = widget.get_allocation()
		window_pos = widget.window.get_position()
		x = window_pos[0] + allocation.x
		y = window_pos[1] + allocation.y + allocation.height
		return (x, y, True)
	
	def on_menu_deactivate(self, widget):
		self.btnSettings.set_sensitive(True)
	
	def on_menuSettings_activate(self, data=None):
		settings_window = settings.Settings()
		settings_window.main()
	
	def on_menuAddBookmark_activate(self, data=None):
		bm_bk_file = open("/home/"+os.getlogin()+"/.pybrowser/bookmark.ini", "r").read()
		bm_file = open("/home/"+os.getlogin()+"/.pybrowser/bookmark.ini", "w")
		bookmark = self.Browser.get_main_frame().get_uri()
		bm_file.write(bm_bk_file + bookmark + "\n")
		bm_file.close()
		
	def on_menuBookmark_activate(self, data=None):
		bookmarks_window = bookmarks.Bookmarks()
		bookmarks_window.main()
		
	###########################
	def main(self):		
		self.wndMain.show_all()
		gtk.main()
'''
class WebView(webkit.WebView):
	def get_html(self):
		self.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		html = self.get_main_frame().get_title()
		self.execute_script('document.title=oldtitle;')
		return html
'''	
	
#start
if __name__ == "__main__":
		if len(argv) == 1:
			App = pyBrowser()
			App.main()
		else:
			url = argv[1]
			App = pyBrowser(url)
			App.main()
			
