#!/usr/bin/python
# -*- coding: latin-1 -*-

##
#   Project: pybrowser - a faster and lightweight web browser written in Python with a GTK+ gui
#   Author: Mario Guerriero <mefrio.g@gmail.com>
#   Copyright: 2011 Mario Guerriero
#   License: GPL-3+
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation; either version 3 of the License, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful, but WITHOUT
#  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
# On Debian GNU/Linux systems, the full text of the GNU General Public License
# can be found in the file /usr/share/common-licenses/GPL-3.

try:
	import pygtk
	import gtk
	import gtk.glade
except:
	print "Install pygtk, gtk and glade modules"
	exit()

try:
	import urllib
	import os
	from sys import argv
	from ConfigParser import RawConfigParser
	from time import sleep
	from shutil import copyfile
except:
	print "Install os, sys, urllib, sleep, shutil and ConfigParser modules but...do you have Python? :-)"  
	exit()

try:
	import webkit
except:
	print "Install webkit module for python"
	exit()
	
import settings	
import bookmarks
import code

class pyBrowser:
	def __init__(self, url=""):
		self.key_sum = ""
		
		def on_wndMain_key_press_event(widget, event, data=None):			
			n = self.notebook.get_current_page()
			scrolled = self.notebook.get_nth_page(n)
			browser = scrolled.get_child()
			
			keyname = gtk.gdk.keyval_name(event.keyval)
			
			self.key_sum += (keyname + "|")

			key_split_sum = self.key_sum.split("|")

			if keyname == "F5":				#Refresh 
				browser.reload()
				
			if keyname == "F11":			#Full screen
				if self.fullscreen == 0:
					self.wndMain.fullscreen()
					self.fullscreen = 1
				elif self.fullscreen == 1:
					self.wndMain.unfullscreen()
					self.fullscreen = 0
			
			if key_split_sum[-2] == "plus" and key_split_sum[-3] == "Control_L":		#set zoom
				browser.zoom_in()
			elif key_split_sum[-2] == "minus" and key_split_sum[-3] == "Control_L":
				browser.zoom_out()
			elif key_split_sum[-2] == "0" and key_split_sum[-3] == "Control_L":
				browser.set_zoom_level(1)

		def on_wndMain_button_press_event(widget, event, data=None):
			#print event
			#button_name = gtk.gdk.button_name(event.button)
			pass
					
		self.MainGlade = gtk.glade.XML("../ui/pybrowser.glade")
		
		gw = self.MainGlade.get_widget
		
		self.wndMain = gw("wndMain")
		self.browView = gw("browView")
		self.entUrl = gw("entUrl")
		self.btnBack = gw("btnBack")
		self.btnForward = gw("btnForward")
		self.imgStopRefresh = gw("imgStopRefresh") 
		self.notebook = gw("notebook")
		
		self.btnSettings = gw("btnSettings")
		self.menu = gw("menu")
				
		self.fullscreen = 0
		
		self.controll_folder()
		
		config = RawConfigParser()
		config.read("/home/"+os.getlogin()+"/.pybrowser/settings.ini")
		self.url = config.get("General", "Homepage")
		
		self.entUrl.set_text(self.url)
		
		self.start_web(self.url)
	
		self.controll_web_status()
		
		signals = {
			"on_wndMain_destroy":self.on_wndMain_destroy,
			"on_btnHome_clicked":self.on_btnHome_clicked,
			"on_btnGoUrl_clicked":self.on_btnGoUrl_clicked,
			"on_btnBack_clicked":self.on_btnBack_clicked,
			"on_btnForward_clicked":self.on_btnForward_clicked,
			"on_btnStopRefresh_clicked":self.on_btnStopRefresh_clicked,
			"on_btnSettings_clicked":self.on_btnSettings_clicked,
			"on_wndMain_key_press_event":on_wndMain_key_press_event,
			"on_wndMain_button_press_event":on_wndMain_button_press_event,
			"on_entUrl_activate":self.on_btnGoUrl_clicked,
			"on_btnAdd_clicked":self.on_btnAdd_clicked,
			"on_menuSettings_activate":self.on_menuSettings_activate,
			"on_menuAddBookmark_activate":self.on_menuAddBookmark_activate,
			"on_menuBookmark_activate":self.on_menuBookmark_activate,
			"on_menuDevCode_activate":self.on_menuDevCode_activate,
			"on_menu_deactivate":self.on_menu_deactivate,
			"on_notebook_switch_page":self.on_notebook_switch_page
		}
		
		self.MainGlade.signal_autoconnect(signals)
	
	#events functions	
	def load_started(self, view, frame):
		self.StopRefresh = 0
		self.wndMain.set_title("Loading...")
		self.notebook.get_current_page()
		self.imgStopRefresh.set_from_stock("gtk-stop", "large-toolbar")

	def load_progress_changed(self, view, frame):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		self.entUrl.set_progress_fraction(browser.get_progress())

	def load_error(self, view, frame, web, data=None):
		self.refresh_window()
		self.set_entry_url()
		self.entUrl.set_progress_fraction(0)
					
	def load_finished(self, view, frame):
		self.StopRefresh = 1
		self.refresh_window()
		self.set_entry_url()
		self.imgStopRefresh.set_from_stock("gtk-refresh", "large-toolbar")
		self.entUrl.set_progress_fraction(1)
		sleep(1)
		self.entUrl.set_progress_fraction(0)

		
	def download_requested(self, view, frame):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		try:
			download_url = browser.get_focused_frame().get_uri()
			file_name = download_url[0:(len(download_url)-1)]
			file_name = file_name.replace("http://", "")
			file_download = file_name.split("/")
			print file_download[-1]
			urllib.urlretrieve(download_url, "/home/" + os.getlogin() + "/" + file_name)
		except:
			print "Cannot download the file"
	
	#window/others settings
	def controll_web_status(self):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		if not browser.can_go_forward():
			self.btnForward.set_sensitive(False)
		else:
			self.btnForward.set_sensitive(True)
	
	def refresh_window(self):
		self.controll_web_status()
		self.set_window_title()
		self.set_entry_url()
		
	def set_window_title(self):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
	
		self.wndMain.set_title( browser.get_main_frame().get_title() + " - pyBrowser" )
		
	
	def set_entry_url(self):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		if browser.get_main_frame().get_uri() != None:
			self.entUrl.set_text( browser.get_main_frame().get_uri() )
		else:
			self.entUrl.set_text( "" )
							
	def start_web(self, url):
		if url[0:7] != "http://":
			url_mod = url
			url = ""
			url = "http://" + url_mod
		elif url[0:7] == "http://":
			pass
			
		self.on_btnAdd_clicked(url)

		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		browser.open(url)
		browser.execute_script('document.title=oldtitle;')


	def controll_folder(self):
		if os.path.exists("/home/"+os.getlogin()+"/.pybrowser"):
			pass
		else:
			os.mkdir("/home/"+os.getlogin()+"/.pybrowser")
			copyfile("../data/settings.ini", "/home/"+os.getlogin()+"/.pybrowser/settings.ini")
			copyfile("../data/bookmark.ini", "/home/"+os.getlogin()+"/.pybrowser/bookmark.ini")
	
	#buttons and window
	def on_wndMain_destroy(self, data=None):
		gtk.main_quit()
			
	def on_btnHome_clicked(self, data=None):
		func = RawConfigParser()
		func.read("/home/"+os.getlogin()+"/.pybrowser/settings.ini")
		home = func.get("General", "Homepage")
		if home[0:7] != "http://":
			home_mod = home
			home = ""
			home = "http://" + home_mod
		elif home[0:7] == "http://":
			pass
		
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		browser.open(home)
		browser.execute_script('document.title=oldtitle;')
		
	def on_btnGoUrl_clicked(self, data=None):
		url = self.entUrl.get_text()
		if url[0:7] != "http://":
			url_mod = url
			url = ""
			url = "http://" + url_mod
		elif url[0:7] == "http://":
			pass
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		browser.execute_script('oldtitle=document.title;document.title=document.documentElement.innerHTML;')
		browser.open(url)
		browser.execute_script('document.title=oldtitle;')
		
	def on_entUrl_activate(self, data=None):
		self.on_btnGoUrl_clicked()
	
	def on_btnBack_clicked(self, data=None):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		browser.go_back()
				
	def on_btnForward_clicked(self, data=None):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		browser.go_forward()
		
	def on_btnStopRefresh_clicked(self, data=None):
		if self.StopRefresh == 0:
			n = self.notebook.get_current_page()
			scrolled = self.notebook.get_nth_page(n)
			browser = scrolled.get_child()
			browser.stop_loading()
			self.controll_web_status()
			
		elif self.StopRefresh == 1:	
			n = self.notebook.get_current_page()
			scrolled = self.notebook.get_nth_page(n)
			browser = scrolled.get_child()
			browser.reload()
			self.controll_web_status()
			
	def on_btnAdd_clicked(self, widget):
		self.notebook.set_show_tabs(True)
		
		new_hbox = gtk.HBox(False,2)
		new_label = gtk.Label("new tab")
		new_button = gtk.Button("X")
		new_hbox.add(new_label)
		new_hbox.add(new_button)
		
		self.notebook.set_tab_reorderable(new_label, True)
		
		new_label.show()
		new_button.show()
		new_hbox.show()
			
		new_scrolled = gtk.ScrolledWindow()
		new_web = webkit.WebView()
		new_web.open("http://www.google.it")

		new_scrolled.add(new_web)
		new_scrolled.show()
		new_web.show()
		
		#new_button.set_expand(False)
		new_button.set_relief(gtk.RELIEF_NONE)
		#new_button.set_expand(False)
				
		self.notebook.append_page(new_scrolled, new_hbox)
		self.notebook.next_page()
		#self.notebook.set_tab_reordeable(new_web, True)
		
		page = self.notebook.get_current_page()
		
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)
		
		new_button.connect("clicked", lambda w: self.remove_book(page))
		new_web.connect("download-requested", self.download_requested)
		new_web.connect("load-finished", self.load_finished)
		new_web.connect("load-started", self.load_started)
		new_web.connect("load-progress-changed", lambda w, x: self.load_progress_changed_tab(new_label))
		new_web.connect("load-progress-changed", self.load_progress_changed)
		new_web.connect("load-error", self.load_error)

	def on_notebook_switch_page(self, widget, notebook, data=None):
		#self.refresh_window()
		self.set_entry_url()
		self.set_window_title()
		
	#tab functions
	def remove_book(self, page):
		self.notebook.remove_page(page)
		# Need to refresh the widget -- 
		# This forces the widget to redraw itself.
		self.notebook.queue_draw_area(0,0,-1,-1)
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)
	
	def load_progress_changed_tab(self, label):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		if browser.get_main_frame().get_title() != None:
			if len( browser.get_main_frame().get_title() ) <= 20:
				label.set_text( browser.get_main_frame().get_title() )
			else:
				label.set_text( browser.get_main_frame().get_title()[0:18] + "..." )
		else:
			label.set_text( "pyBrowser" )
		
	#######	
	
	def on_btnSettings_clicked(self, widget):
		if not self.menu.get_attach_widget():
			self.menu.attach_to_widget(widget, None)
		self.menu.popup(None, None, self.menu_position, 1, gtk.get_current_event_time())
		self.menu.select_first(True)
		self.btnSettings.set_sensitive(False)

	
	#functions for menu
	def menu_position(self, menu, data=None):
		widget = menu.get_attach_widget()
		allocation = widget.get_allocation()
		window_pos = widget.window.get_position()
		x = window_pos[0] + allocation.x
		y = window_pos[1] + allocation.y + allocation.height
		return (x, y, True)
	
	def on_menu_deactivate(self, widget):
		self.btnSettings.set_sensitive(True)
	
	def on_menuSettings_activate(self, data=None):
		settings_window = settings.Settings()
		settings_window.main()
	
	def on_menuAddBookmark_activate(self, data=None):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		
		bm_bk_file = open("/home/"+os.getlogin()+"/.pybrowser/bookmark.ini", "r").read()
		bm_file = open("/home/"+os.getlogin()+"/.pybrowser/bookmark.ini", "w")
		bookmark = browser.get_main_frame().get_uri()
		title = browser.get_main_frame().get_title()
		bm_file.write(bm_bk_file + bookmark + "-" + title + "\n")
		bm_file.close()
		
	def on_menuDevCode_activate(self, data=None):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
				
		code_window = code.Code( browser.get_main_frame().get_uri() )
		code_window.main()
		
	def on_menuBookmark_activate(self, data=None):
		bookmarks_window = bookmarks.Bookmarks()
		bookmarks_window.main()
		
	###########################
	def main(self):		
		self.wndMain.show_all()
		gtk.main()

	
#start
if __name__ == "__main__":
		if len(argv) == 1:
			App = pyBrowser()
			App.main()
		else:
			url = argv[1]
			App = pyBrowser(url)
			App.main()
			
