import webkit
import gtk

from ConfigParser import RawConfigParser

import about

from controller import *

config = RawConfigParser()
config.read(DATA_DIR + "settings.ini")
HOME = config.get("General", "Homepage")

class WebView(webkit.WebView):
	def __init__(self, main_window, btnback, btnforward, btnrefresh, entryurl, notebook_tab):
		super(WebView, self).__init__()
		
		self.wnd = wnd(main_window)
		self.btn_back = back(btnback)
		self.btn_forward = forward(btnforward)
		self.btn_refresh = refresh(btnrefresh)
		self.entry = entry(entryurl)
		self.notebook = notebook(notebook_tab)
		
		self.hovering = None
		
	def signals_connect(self, view):
		view.connect("load-started", self.load_started)
		view.connect("load-progress-changed", self.load_progress_changed)
		view.connect("load-finished", self.load_finished)
		view.connect("title-changed", self.title_changed)
		view.connect_after("populate-popup", self.populate_popup)
		view.connect("hovering-over-link", self.hovering_over_link)

	"""generic functions"""
	def start(self, url):
		view = self.get_current_view()
		view.open(self.get_url_protocol(url))
				
	def back(self):
		view = self.get_current_view()
		view.go_back()
		
	def forward(self):
		view = self.get_current_view()
		view.go_forward()
		
	def stop_or_refresh(self):
		view = self.get_current_view()
		
		if self.StopRefresh == 0:
			view.stop_loading()
			self.set_back_forward()
						
		elif self.StopRefresh == 1:	
			view.reload()
			self.set_back_forward()
			
	def code_view(self, widget):
		view = self.get_current_view()
		
		if view.get_view_source_mode() == False:	
			view.set_view_source_mode(True)
			view.reload()		
		else:
			view.set_view_source_mode(False)
			view.reload()
			
	"""functions for the tab"""
	def new_tab(self, widget, url):
		"""creating of a new tab with label, close button and webview"""
		self.notebook.set_show_tabs(True)
		self.notebook.set_scrollable(True)
		
		new_hbox = gtk.HBox(False, 2)
		new_label = gtk.Label("New tab")
		new_button = gtk.Button("X")
		new_hbox.add(new_label)
		new_hbox.add(new_button)
				
		self.notebook.set_tab_reorderable(new_label, True)
		
		new_label.show()
		new_button.show()
		new_hbox.show()
			
		new_scrolled = gtk.ScrolledWindow()
		new_web = webkit.WebView()
		new_web.open(self.get_url_protocol(url))

		new_scrolled.add(new_web)
		new_scrolled.show()
		new_web.show()
		
		new_button.set_relief(gtk.RELIEF_NONE)
				
		self.notebook.append_page(new_scrolled, new_hbox)
		self.notebook.next_page()
		
		page = self.notebook.get_current_page()
		
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)	

		new_button.connect("clicked", lambda w: self.remove_tab(page))
		
		self.signals_connect(new_web)
	
	def remove_tab(self, page):
		"""function associated with the click of the button X on the tab. It is used to close a tab"""
		self.notebook.remove_page(page)
		self.notebook.queue_draw_area(0,0,-1,-1)
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)
			
	def switch_tab(self, widget, tab, data=None):
		"""functions associated to the switching of the tabs"""
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n-1)
		view = scrolled.get_child()
		
		if view.get_main_frame().get_title() != None:
			self.wnd.set_title( view.get_main_frame().get_title() + " - pyBrowser" )
		else:
			self.wnd.set_title( "pyBrowser" )
		
		if not view.can_go_forward():
			self.btn_forward.set_sensitive(False)
		else:
			self.btn_forward.set_sensitive(True)
		
		if not view.can_go_back():
			self.btn_back.set_sensitive(False)
		else:
			self.btn_back.set_sensitive(True)
		
		if view.get_main_frame().get_uri() != None:
			self.entry.set_text( view.get_main_frame().get_uri() )
		else:
			self.entry.set_text( "" )

	"""functions for the signals"""
	def load_started(self, view, frame):
		self.StopRefresh = 0
		self.wnd.set_title("Loading...")
		self.btn_refresh.set_from_stock("gtk-stop", "large-toolbar")
	
	def load_progress_changed(self, view, frame):
		view = self.get_current_view()
		self.set_back_forward()
		self.set_entry_url()
		self.entry.set_progress_fraction(view.get_progress())
		self.set_tab_label()
		
	def load_finished(self, view, frame):
		self.StopRefresh = 1
		self.set_window_title()
		self.set_entry_url()
		self.entry.set_progress_fraction(0)
		self.btn_refresh.set_from_stock("gtk-refresh", "large-toolbar")
		
	def title_changed(self, title, view, frame):
		self.set_entry_url()
		self.set_window_title()
		self.set_back_forward()
		"""history"""
		view = self.get_current_view()
		hs_bk_file = open(DATA_DIR + "history.ini", "r").read()
		hs_file = open(DATA_DIR + "history.ini", "w")
		history = str( view.get_main_frame().get_uri() )
		title = str( view.get_main_frame().get_title() )
		hs_file.write(hs_bk_file + history + "|" + title + "\n")
		hs_file.close()

		
	def populate_popup(self, view, menu):
		"""this function is used when appear the menu of the right click on the webview"""
		view = self.get_current_view()
				
		zoom_in = gtk.ImageMenuItem(gtk.STOCK_ZOOM_IN)
		zoom_in.connect('activate', lambda w, i: view.zoom_in(), view)
		menu.append(zoom_in)

		zoom_out = gtk.ImageMenuItem(gtk.STOCK_ZOOM_OUT)
		zoom_out.connect('activate', lambda w, i: view.zoom_out(), view)
		menu.append(zoom_out)

		zoom_hundred = gtk.ImageMenuItem(gtk.STOCK_ZOOM_100)
		zoom_hundred.connect('activate', lambda w, i: view.set_zoom_level(1), view)
		menu.append(zoom_hundred)
		printitem = gtk.ImageMenuItem(gtk.STOCK_PRINT)
		menu.append(printitem)
		printitem.connect('activate', self.print_cb, view)

		menu.append(gtk.SeparatorMenuItem())

		aboutitem = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
		menu.append(aboutitem)
		about_window = about.About()
		aboutitem.connect('activate', lambda w, i: about_window.main(), view)
	
		if self.hovering != None:
			open_in_new_tab = gtk.MenuItem("Open Link in New Tab")
			open_in_new_tab.connect("activate", lambda w, i: self.new_tab(url=self.hovering, widget=self.notebook), view)
			menu.insert(open_in_new_tab, 0)

		menu.show_all()
		return False
			
	def hovering_over_link(self, view, title, uri):
		self.hovering = uri
				
	"""generic functions"""
	def set_window_title(self):
		view = self.get_current_view()
		
		if view.get_main_frame().get_title() != None:
			self.wnd.set_title( view.get_main_frame().get_title() + " - pyBrowser" )
		else:
			self.wnd.set_title( "pyBrowser" )
			
	def set_back_forward(self):
		view = self.get_current_view()
		
		if not view.can_go_forward():
			self.btn_forward.set_sensitive(False)
		else:
			self.btn_forward.set_sensitive(True)
		
		if not view.can_go_back():
			self.btn_back.set_sensitive(False)
		else:
			self.btn_back.set_sensitive(True)
			
	def set_entry_url(self):
		view = self.get_current_view()
		
		if view.get_main_frame().get_uri() != None:
			self.entry.set_text( view.get_main_frame().get_uri() )
		else:
			self.entry.set_text( "" )
			
	def set_tab_label(self):
		view = self.get_current_view()
		
		label = self.notebook.get_tab_label( self.notebook.get_nth_page( self.notebook.get_current_page() ) ).get_children()

		if view.get_main_frame().get_title() != None:
			if len( view.get_main_frame().get_title() ) <= 20:
				label[0].set_text( view.get_main_frame().get_title() )
			else:
				label[0].set_text( view.get_main_frame().get_title()[0:18] + "..." )
		else:
			label[0].set_text( "pyBrowser" )
			
	def print_cb(self, menu_item, web_view):
		"""function to print the web page"""
		view = self.get_current_view()
		
		mainframe = view.get_main_frame()
		mainframe.print_full(gtk.PrintOperation(), gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG);
		
	"""get the showing webview"""
	def get_current_view(self):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		return browser
		
	def get_url_protocol(self, url):
		url = "http://"+url.split("http://")[-1]
		return url
			
	
def wnd(wnd):
	return wnd
	
def back(back):
	return back

def forward(forward):
	return forward
	
def refresh(refresh):
	return refresh
	
def entry(entry):
	return entry
	
def notebook(notebook):
	return notebook
