import gtk

from ConfigParser import RawConfigParser

import webview
import menu_bookmark
import menu_setting

from controller import *

class Window(gtk.Window):
	def __init__(self):
		super(Window, self).__init__()
		"""some variables used in the following code"""
		self.get_fullscreen = 0
		self.key_sum = ""
		"""window"""
		self.set_title("pyBrowser")
		self.set_icon_from_file("../icons/pybrowser.png")
		self.set_default_size(700,550)
		self.maximize()
		
		self.get_user_settings()
		
		"""create and and draw all the widgets"""
		self.create_widgets()
		self.draw()
		
		"""signals"""
		self.signals_connect()
		
		"""send the widget"""
		self.send_widget()

	def create_widgets(self):
		self.vbox = gtk.VBox(False, 0)
		self.hbox = gtk.HBox(False, 0)
		"""buttons and entry for the hbox"""
		img = gtk.Image()
		img.set_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_LARGE_TOOLBAR)		
		self.go_back = gtk.Button()
		self.go_back.set_image(img)
		self.go_back.set_relief(gtk.RELIEF_NONE)		
		
		img = gtk.Image()
		img.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.go_forward = gtk.Button()
		self.go_forward.set_image(img)
		self.go_forward.set_relief(gtk.RELIEF_NONE)		
		
		img = gtk.Image()
		img.set_from_stock(gtk.STOCK_HOME, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.go_home = gtk.Button()
		self.go_home.set_image(img)
		self.go_home.set_relief(gtk.RELIEF_NONE)
		
		self.img_refresh = gtk.Image()
		self.img_refresh.set_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.refresh_stop = gtk.Button()
		self.refresh_stop.set_image(self.img_refresh)
		self.refresh_stop.set_relief(gtk.RELIEF_NONE)
		
		self.entry_url = gtk.Entry()
		
		img = gtk.Image()
		img.set_from_file('../icons/bookmark.svg')
		self.bookmarks = gtk.Button()
		self.bookmarks.set_image(img)
		self.bookmarks.set_relief(gtk.RELIEF_NONE)
	
		img = gtk.Image()
		img.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.add_tab = gtk.Button()
		self.add_tab.set_image(img)
		self.add_tab.set_relief(gtk.RELIEF_NONE)
	
		img = gtk.Image()
		img.set_from_stock(gtk.STOCK_PROPERTIES, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.settings = gtk.Button()
		self.settings.set_image(img)
		self.settings.set_relief(gtk.RELIEF_NONE)
		
		"""notebook"""
		self.notebook = gtk.Notebook()
		self.notebook.set_show_tabs(False)
		
		hbox = gtk.HBox(False,2)
		label = gtk.Label("New tab")
		button = gtk.Button("X")
		hbox.add(label)
		hbox.add(button)
		
		#THE FOLLOW LINE DEFINE THE CLASS FOR THE WEBVIEW, THE HEARTH OF THIS BROWSER
		self.web = webview.WebView(self, self.go_back, self.go_forward, self.img_refresh, self.entry_url, self.notebook)
		#THIS LINE IS FOR THE BOOKMARKS MENU
		self.menuBookmark = menu_bookmark.MenuBookmark(self.bookmarks, self.web) 	
		
		self.web.new_tab(url=self.HOME, widget=self.notebook)
		
		#THIS LINE IS FOR THE SETTINGS MENU
		self.menuSetting = menu_setting.MenuSetting(self.settings, self.web) 	 
			
	def draw(self):
		self.hbox.pack_start(self.go_back, False, False)
		self.hbox.pack_start(self.go_forward, False, False)
		self.hbox.pack_start(self.go_home, False, False)
		self.hbox.pack_start(self.refresh_stop, False, False)
		self.hbox.pack_start(self.entry_url)
		self.hbox.pack_start(self.bookmarks, False, False)
		self.hbox.pack_start(self.add_tab, False, False)
		self.hbox.pack_start(self.settings, False, False)
			
		self.hbox.set_homogeneous(False)
			
		self.vbox.pack_start(self.hbox, False)
		self.vbox.pack_start(self.notebook, True)
		
		self.add(self.vbox)		
	
	def signals_connect(self):
		"""window"""
		self.connect("destroy", gtk.main_quit)
		self.connect("key-press-event", self.key_press_event)
		"""buttons and others"""
		self.go_home.connect("clicked", lambda w: self.web.start(self.HOME))
		self.go_back.connect("clicked", lambda w: self.web.back())
		self.go_forward.connect("clicked", lambda w: self.web.forward())
		self.refresh_stop.connect("clicked", lambda w: self.web.stop_or_refresh())
		self.entry_url.connect("activate", lambda w: self.web.start(self.entry_url.get_text()))
		self.bookmarks.connect("clicked", lambda w: self.menuBookmark.show())
		self.add_tab.connect("clicked", lambda w: self.web.new_tab(self.notebook, self.HOME))
		self.settings.connect("clicked", lambda w: self.menuSetting.show())
		self.notebook.connect("switch-page", self.web.switch_tab)
	
	"""signals functions"""
	def key_press_event(self, widget, event):
		keyval = gtk.gdk.keyval_name(event.keyval)
		view = self.web.get_current_view()	
		
		self.key_sum += (keyval + "|")
		key_split_sum = self.key_sum.split("|")	
					
		if keyval == "F5":
			self.web.get_current_view().reload()
			
		if keyval == "F11":
			if self.get_fullscreen == 0:
				self.fullscreen()
				self.get_fullscreen = 1
			elif self.get_fullscreen == 1:
				self.unfullscreen()
				self.get_fullscreen = 0
				
		if key_split_sum[-2] == "plus" and key_split_sum[-3] == "Control_L":		#set zoom
			view.zoom_in()
		elif key_split_sum[-2] == "minus" and key_split_sum[-3] == "Control_L":
			view.zoom_out()
		elif key_split_sum[-2] == "0" and key_split_sum[-3] == "Control_L":
			view.set_zoom_level(1)
				
	
	def new_tab(self, widget):
		hbox = gtk.HBox(False,2)
		label = gtk.Label("New tab")
		button = gtk.Button("X")
		hbox.add(label)
		hbox.add(button)
		
		#THE FOLLOW LINE DEFINE THE CLASS FOR THE WEBVIEW, THE HEARTH OF THIS BROWSER
		web = webview.WebView(self, self.go_back, self.go_forward, self.img_refresh, self.entry_url, self.notebook)	
		
		scrolled = gtk.ScrolledWindow()
		view = web.start(self.HOME)
		scrolled.add(view)
		self.notebook.append_page(scrolled, hbox)
		
	
	"""this function is used to send to the other files the necessary gtk widgets"""
	def send_widget(self):
		"""to webview file"""
		webview.wnd(self)
		webview.back(self.go_back)
		webview.forward(self.go_forward)
		webview.refresh(self.refresh_stop)
		webview.entry(self.entry_url)
		webview.notebook(self.notebook)
		"""to menu_bookmark file"""
		menu_bookmark.btn(self.bookmarks)
		menu_bookmark.view(self.web)
		"""to menu_setting file"""
		menu_setting.btn(self.bookmarks)
		menu_setting.web(self.web)		
	
	"""get the user settings"""
	def get_user_settings(self):
		config = RawConfigParser()
		config.read(DATA_DIR + "settings.ini")
		self.HOME = config.get("General", "Homepage")
		self.DOWNLOAD_PATH = config.get("General", "DownloadPath")
		
	def main(self):
		self.show_all()
		gtk.main()

if __name__ == "__main__": 
	wnd = Window()
	wnd.main()
