#/usr/bin/env python
##
#   Project: pybrowser - A fast and lightweight browser  
#   Author: Mario Guerriero <mefrio.g@gmail.com>
#   Copyright: 2011 Mario Guerriero
#   License: GPL-3+
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation; either version 3 of the License, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful, but WITHOUT
#  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
# On Debian GNU/Linux systems, the full text of the GNU General Public License
# can be found in the file /usr/share/common-licenses/GPL-3.
##

from gi.repository import WebKit as webkit
from gi.repository import Gtk
from gi.repository import Pango

from ConfigParser import RawConfigParser

import about
import download

from controller import *

from translate import _

controll_folder()
config = RawConfigParser()
config.read(SETTINGS_FILE)
HOME = config.get("General", "Homepage")

class WebView(webkit.WebView):
	def __init__(self, main_window, btnback, btnforward, btnrefresh, entryurl, notebook_tab):
		super(WebView, self).__init__()
		
		self.show_tabs_opt = False
		
		self.wnd = wnd(main_window)
		self.btn_back = back(btnback)
		self.btn_forward = forward(btnforward)
		self.btn_refresh = refresh(btnrefresh)
		self.entry = entry(entryurl)
		self.notebook = notebook(notebook_tab)
		
		self.hovering = None
		
	def signals_connect(self, view):
		view.connect("load-started", self.load_started)
		view.connect("load-progress-changed", self.load_progress_changed)
		view.connect("load-finished", self.load_finished)
		view.connect("title-changed", self.title_changed)
		view.connect_after("populate-popup", self.populate_popup)
		view.connect("hovering-over-link", self.hovering_over_link)
		view.connect("download-requested", self.download_requested)

	"""generic functions"""
	def start(self, url):
		view = self.get_current_view()
		view.open(self.get_url_protocol(url))
				
	def back(self):
		view = self.get_current_view()
		view.go_back()
		
	def forward(self):
		view = self.get_current_view()
		view.go_forward()
		
	def stop_or_refresh(self):
		view = self.get_current_view()
		
		if self.StopRefresh == 0:
			view.stop_loading()
			self.set_back_forward()
						
		elif self.StopRefresh == 1:	
			view.reload()
			self.set_back_forward()
			
	def code_view(self, widget):
		view = self.get_current_view()
		
		if view.get_view_source_mode() == False:	
			view.set_view_source_mode(True)
			view.reload()		
		else:
			view.set_view_source_mode(False)
			view.reload()
			
	"""functions for the tab"""
	def new_tab(self, widget, url):
		"""creating of a new tab with label, close button and webview"""
		self.notebook.set_show_tabs(True)
		self.notebook.set_scrollable(True)
		
		import widgets.tablabel as widgets
		
		self.new_hbox = widgets.TabLabel()
		new_label = Gtk.Label(label=_("New tab"))
		new_label.props.max_width_chars = 30
		new_label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
		new_label.set_alignment(0.0, 0.5)
		new_button = Gtk.Button()
		icon = Gtk.Image.new_from_stock(Gtk.STOCK_ORIENTATION_PORTRAIT, Gtk.IconSize.BUTTON)
		close_image = Gtk.Image.new_from_stock(Gtk.STOCK_CLOSE, Gtk.IconSize.MENU)
		new_button.set_relief(Gtk.ReliefStyle.NONE)
		new_button.set_image(close_image)
		self.new_hbox.pack_start(icon, False, False, 0)
		self.new_hbox.pack_start(new_label, True, True, 0)
		self.new_hbox.pack_start(new_button, False, False, 0)
        
		self.new_hbox.set_data("label", new_label)
		self.new_hbox.set_data("close-button", new_button)
		self.new_hbox.connect("style-set", self.tab_label_style_set)
				
		#self.new_hbox.connect("close", self.remove_tab)
		self.new_hbox.show_all()
				
		self.notebook.set_tab_reorderable(new_label, True)
			
		new_scrolled = Gtk.ScrolledWindow()
		new_web = webkit.WebView()
		new_web.open(self.get_url_protocol(url))

		new_scrolled.add(new_web)
		new_scrolled.show()
		new_web.show()
		
		new_button.set_relief(Gtk.ReliefStyle.NONE)
				
		self.notebook.append_page(new_scrolled, self.new_hbox)
		self.notebook.next_page()
		
		page = self.notebook.get_current_page()
		
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)	

		new_button.connect("clicked", self.remove_tab, new_scrolled)
		#new_button.connect("clicked", lambda w: self.remove_tab(self.notebook.get_current_page()))
		
		self.signals_connect(new_web)
		
	def tab_label_style_set(self, tab_label, style):
		"""set the tab label style"""
		context = tab_label.get_pango_context()
		metrics = context.get_metrics(tab_label.style.font_desc, context.get_language())
		char_width = metrics.get_approximate_digit_width()
		(width, height) = Gtk.icon_size_lookup(Gtk.IconSize.MENU)
		tab_label.set_size_request(20 * Pango.PIXELS(char_width) + 2 * width, Pango.PIXELS(metrics.get_ascent() + metrics.get_descent()) + 8)
	
	def remove_tab(self, label, child):
		"""function associated with the click of the button X on the tab. It is used to close a tab"""
		page_num = self.notebook.page_num(child)
		if page_num != -1:
			view = child.get_child()
			view.destroy()
			self.notebook.remove_page(page_num)
		if self.notebook.get_n_pages() == 1:
			self.notebook.set_show_tabs(False)
			
	def switch_tab(self, widget, tab, data=None):
		"""functions associated to the switching of the tabs"""
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n-1)
		view = scrolled.get_child()
		
		if view.get_main_frame().get_title() != None:
			self.wnd.set_title( view.get_main_frame().get_title() + " - pyBrowser" )
		else:
			self.wnd.set_title( "pyBrowser" )
		
		if not view.can_go_forward():
			self.btn_forward.set_sensitive(False)
		else:
			self.btn_forward.set_sensitive(True)
		
		if not view.can_go_back():
			self.btn_back.set_sensitive(False)
		else:
			self.btn_back.set_sensitive(True)
		
		if view.get_main_frame().get_uri() != None:
			self.entry.set_text( view.get_main_frame().get_uri() )
		else:
			self.entry.set_text( "" )

	"""functions for the signals"""
	def load_started(self, view, frame):
		self.StopRefresh = 0
		self.wnd.set_title(_("Loading..."))
		self.btn_refresh.set_from_stock("Gtk-stop", "large-toolbar")
	
	def load_progress_changed(self, view, frame):
		view = self.get_current_view()
		self.set_back_forward()
		self.set_entry_url()
		self.entry.set_progress_fraction(view.get_progress())
		self.set_tab_label()
		
	def load_finished(self, view, frame):
		self.StopRefresh = 1
		self.set_window_title()
		self.set_entry_url()
		self.entry.set_progress_fraction(0)
		self.btn_refresh.set_from_stock("Gtk-refresh", "large-toolbar")
		
	def title_changed(self, title, view, frame):
		self.set_entry_url()
		self.set_window_title()
		self.set_back_forward()
		"""history"""
		view = self.get_current_view()
		hs_bk_file = open(HISTORY_FILE, "r").read()
		hs_file = open(HISTORY_FILE, "w")
		history = str( view.get_main_frame().get_uri() )
		title = str( view.get_main_frame().get_title() )
		hs_file.write(hs_bk_file + history + "|" + title + "\n")
		hs_file.close()

		
	def populate_popup(self, view, menu):
		"""this function is used when appear the menu of the right click on the webview"""
		view = self.get_current_view()
				
		zoom_in = Gtk.ImageMenuItem(Gtk.STOCK_ZOOM_IN)
		zoom_in.connect('activate', lambda w, i: view.zoom_in(), view)
		menu.append(zoom_in)

		zoom_out = Gtk.ImageMenuItem(Gtk.STOCK_ZOOM_OUT)
		zoom_out.connect('activate', lambda w, i: view.zoom_out(), view)
		menu.append(zoom_out)

		zoom_hundred = Gtk.ImageMenuItem(Gtk.STOCK_ZOOM_100)
		zoom_hundred.connect('activate', lambda w, i: view.set_zoom_level(1), view)
		menu.append(zoom_hundred)
		printitem = Gtk.ImageMenuItem(Gtk.STOCK_PRINT)
		menu.append(printitem)
		printitem.connect('activate', self.print_cb, view)

		menu.append(Gtk.SeparatorMenuItem())

		aboutitem = Gtk.ImageMenuItem(Gtk.STOCK_ABOUT)
		menu.append(aboutitem)
		about_window = about.About()
		aboutitem.connect('activate', lambda w, i: about_window.main(), view)
	
		if self.hovering != None:
			open_in_new_tab = Gtk.MenuItem(_("Open Link in New Tab"))
			open_in_new_tab.connect("activate", lambda w, i: self.new_tab(url=self.hovering, widget=self.notebook), view)
			menu.insert(open_in_new_tab, 0)

		menu.show_all()
		return False
			
	def hovering_over_link(self, view, title, uri):
		self.hovering = uri
				
	def download_requested(self, view, frame):
		download.start_download(self.hovering)		
		
	"""generic functions"""
	def set_window_title(self):
		view = self.get_current_view()
		
		if view.get_main_frame().get_title() != None:
			self.wnd.set_title( view.get_main_frame().get_title() + " - pyBrowser" )
		else:
			self.wnd.set_title( "pyBrowser" )
			
	def set_back_forward(self):
		view = self.get_current_view()
		
		if not view.can_go_forward():
			self.btn_forward.set_sensitive(False)
		else:
			self.btn_forward.set_sensitive(True)
		
		if not view.can_go_back():
			self.btn_back.set_sensitive(False)
		else:
			self.btn_back.set_sensitive(True)
			
	def set_entry_url(self):
		view = self.get_current_view()
		
		if view.get_main_frame().get_uri() != None:
			self.entry.set_text( view.get_main_frame().get_uri() )
		else:
			self.entry.set_text( "" )
			
	def set_tab_label(self):
		view = self.get_current_view()
		
		label = self.notebook.get_tab_label( self.notebook.get_nth_page( self.notebook.get_current_page() ) ).get_children()

		if view.get_main_frame().get_title() != None:
			if len( view.get_main_frame().get_title() ) <= 20:
				label[1].set_text( view.get_main_frame().get_title() )
			else:
				label[1].set_text( view.get_main_frame().get_title()[0:18] + "..." )
		else:
			label[1].set_text( "pyBrowser" )
			
	def print_cb(self, menu_item, web_view):
		"""function to print the web page"""
		view = self.get_current_view()
		
		mainframe = view.get_main_frame()
		mainframe.print_full(Gtk.PrintOperation(), Gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG);
		
	"""get the showing webview"""
	def get_current_view(self):
		n = self.notebook.get_current_page()
		scrolled = self.notebook.get_nth_page(n)
		browser = scrolled.get_child()
		return browser
		
	def get_url_protocol(self, url):
		url = "http://"+url.split("http://")[-1]
		return url
			
	
def wnd(wnd):
	return wnd
	
def back(back):
	return back

def forward(forward):
	return forward
	
def refresh(refresh):
	return refresh
	
def entry(entry):
	return entry
	
def notebook(notebook):
	return notebook
